/** Copyright (C) 2013 Alexander Mariel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Autor: Alexander Mariel
 * Trabajo para la asignatura TAIA
 * Practica individual con agentes en la plataforma JADE 
 */
package taia.pli.agents;

import jade.core.behaviours.WakerBehaviour;
import taia.pli.behaviours.PlayMusicBehaviour;
import taia.pli.behaviours.ReceiveDirectionsBehaviour;
import taia.pli.behaviours.setupPerformance.SetupPerformanceBehaviour;
import taia.pli.utilities.AudioPlayer;
import taia.pli.utilities.DFRegistrar;

/**
 * This agent represents a bassist
 * 
 * It must be managed with a LeadAgent and provided with BPM from
 * a DrumsAgent
 * 
 * @author Alex
 *
 */
@SuppressWarnings("serial")
public class BassAgent extends MusicianAgent
{
	//Bassist objects to play audio clips
	private AudioPlayer apBassFull;
	private AudioPlayer apBassIntro;
	
	//Variable indicating which clip should be played next
	private int nextPartToPlay;

	protected void setup()
	{
		//Greeting
		System.out.println(getLocalName() + ": Me pregunto si alguien sera capaz de oirme...");

		//Variable initialization
		nextPartToPlay = 0;
		apBassFull = new AudioPlayer("/res/bass_full.wav");
		apBassIntro = new AudioPlayer("/res/bass_intro.wav");

		//Register agent in DF as music player and bassist 
		String[] services = { DFRegistrar.MUSICO, DFRegistrar.BASS };
		DFRegistrar.getMyDFR().registerInDF(this, services);

		//Delay behaviour add because of reasons
		addBehaviour(new WakerBehaviour(this, 8000) 
		{
			protected void onWake()
			{
				//Add behaviours
				//Setup needed material to paly
				addBehaviour(new SetupPerformanceBehaviour());
				//Receive DrumsAgetn ticks and play music to the rythm
				addBehaviour(new PlayMusicBehaviour());
				//Be able to listen for instructions form the LeadAgent
				addBehaviour(new ReceiveDirectionsBehaviour());
			}
		});		
	}
	
	//Set next clip to be played
	public void setNextPartToPlay(int pNum)
	{
		nextPartToPlay = pNum;
	}

	protected void takeDown()
	{
		//De-Register agent form DF, as it will die in the near future
		DFRegistrar.getMyDFR().deRegisterFromDF(this);
	}

	@Override
	public void play()
	{
		//Depending on which aprt of the song to play
		switch(nextPartToPlay)
		{
			case 0:
				//Dont paly anything
				break;
			
			//Play a clip				
			case 1:
				apBassIntro.startSound();
				break;
				
			case 2:
				apBassFull.startSound();
				break;
				
			default: 
				break;
		}		
	}
}
