/** Copyright (C) 2013 Alexander Mariel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Autor: Alexander Mariel
 * Trabajo para la asignatura TAIA
 * Practica individual con agentes en la plataforma JADE 
 */
package taia.pli.agents;

import jade.core.behaviours.WakerBehaviour;
import taia.pli.behaviours.MetronomeBehaviour;
import taia.pli.behaviours.PlayMusicBehaviour;
import taia.pli.behaviours.ReceiveDirectionsBehaviour;
import taia.pli.behaviours.setupPerformance.SetupPerformanceBehaviour;
import taia.pli.utilities.AudioPlayer;
import taia.pli.utilities.DFRegistrar;

/**
 * This agent represents a drummer.
 * 
 * It sends the beat signal to the rest of agents through its
 * MetronomeBehaviour behaviour.
 * It listens to its own beat signal, plays music and listens
 * for the lead agents directions
 * 
 * Non-documented variables and methods can be looked up in BassAgent
 * 
 * @author Alex
 *
 */
@SuppressWarnings("serial")
public class DrumsAgent extends MusicianAgent
{	
	//Time between beats, it is important to avoid clipping between sound clips
	public static final int MAGIC_NUM = 4800;
	
	private AudioPlayer apBatFull;
	private AudioPlayer apBatMid;
	
	private int nextPartToPlay;
	
	protected void setup()
	{
		System.out.println(getLocalName()+": Ya se me han vuelto a caer las baquetas...");
		
		nextPartToPlay = 0;
		apBatFull = new AudioPlayer("/res/bat_full_desc.wav");
		apBatMid = new AudioPlayer("/res/bat_mid_desc.wav");
		
		String[] services = {DFRegistrar.MUSICO, DFRegistrar.DRUMS};
		
		DFRegistrar.getMyDFR().registerInDF(this, services);
				
		addBehaviour(new WakerBehaviour(this, 8000) 
		{
			protected void onWake()
			{
				addBehaviour(new SetupPerformanceBehaviour());
				addBehaviour(new PlayMusicBehaviour());
				addBehaviour(new ReceiveDirectionsBehaviour());
			}
		});	
		
		addBehaviour(new WakerBehaviour(this, 15000) 
		{
			protected void onWake()
			{
				//Adds the magic behaviour which moves the band
				addBehaviour(new MetronomeBehaviour(myAgent));
			}
		});	
		
	}
	
	public void setNextPartToPlay(int pNum)
	{
		nextPartToPlay = pNum;
	}
	
	protected void takeDown()
	{
		DFRegistrar.getMyDFR().deRegisterFromDF(this);
	}

	@Override
	public void play()
	{		
		switch(nextPartToPlay)
		{
			case 0:
				break;
			
			case 1:
				apBatMid.startSound();
				break;
				
			case 2:
				apBatFull.startSound();
				break;
				
			default: 
				break;
		}	
	}
}
