/** Copyright (C) 2013 Alexander Mariel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Autor: Alexander Mariel
 * Trabajo para la asignatura TAIA
 * Practica individual con agentes en la plataforma JADE 
 */
package taia.pli.agents;

import jade.core.behaviours.WakerBehaviour;
import taia.pli.behaviours.DirectBandBehaviour;
import taia.pli.behaviours.LeadPlayMusicBehaviour;
import taia.pli.behaviours.NotifyStartPerformanceBehaviour;
import taia.pli.behaviours.ReceiveDirectionsBehaviour;
import taia.pli.behaviours.setupPerformance.SetupPerformanceBehaviour;
import taia.pli.utilities.AudioPlayer;
import taia.pli.utilities.DFRegistrar;

/**
 * This agent represents a lead (in musical and team sense).
 * 
 * It listens for the beat signal, plays music and listens
 * for his own lead agents directions
 * 
 * This agent directs the whole band, telling everyone what to play.
 * It also manages the performance setup, making everyone wait till everyone
 * is ready
 * 
 * Non-documented variables and methods can be looked up in BassAgent
 * 
 * @author Alex
 *
 */
@SuppressWarnings("serial")
public class LeadAgent extends MusicianAgent
{
	//Constant strings to be used in ACL messages
	public static final String START = "Comenzamos";
	public static final String READY = "Estoy listo";
	public static final String DIRECTION_MSG = "direccion";
	
	private AudioPlayer apLeadFullv1;
	private AudioPlayer apLeadFullv2;
	private AudioPlayer apLeadRiff;
	private int nextPartToPlay;
	
	protected void setup()
	{
		System.out.println(getLocalName()+": Les voy a dejar impresionados!");
		
		nextPartToPlay = 0;
		
		apLeadFullv1 = new AudioPlayer("/res/lead_full_v1.wav");
		apLeadFullv2 = new AudioPlayer("/res/lead_full_v2.wav");
		apLeadRiff = new AudioPlayer("/res/lead_riff.wav");
		
		String[] services = {DFRegistrar.MUSICO, DFRegistrar.LEAD};
		
		DFRegistrar.getMyDFR().registerInDF(this, services);
		
		addBehaviour(new NotifyStartPerformanceBehaviour());

		addBehaviour(new WakerBehaviour(this, 5000) 
		{
			protected void onWake()
			{
				addBehaviour(new SetupPerformanceBehaviour());
			}
		});
		
		addBehaviour(new WakerBehaviour(this, 13000) 
		{
			protected void onWake()
			{
				addBehaviour(new LeadPlayMusicBehaviour());
				addBehaviour(new DirectBandBehaviour());
				addBehaviour(new ReceiveDirectionsBehaviour());
			}
		});
		
	}
	
	public void setNextPartToPlay(int pNum)
	{
		//As this agents clips are a bit different
		//I cheat to make it sound better
		if((nextPartToPlay == 1)&&(pNum == 1))
		{
			nextPartToPlay = 3;
		}
		
		nextPartToPlay = pNum;
	}
	
	protected void takeDown()
	{
		DFRegistrar.getMyDFR().deRegisterFromDF(this);
	}

	@Override
	public void play()
	{
		switch(nextPartToPlay)
		{
			case 0:
				break;
			
			case 1:
				apLeadFullv1.startSound();
				break;
				
			case 2:
				apLeadRiff.startSound();
				break;
				
			case 3:
				apLeadFullv2.startSound();
				break;
				
			default: 
				break;
		}	
	}
}
