/** Copyright (C) 2013 Alexander Mariel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Autor: Alexander Mariel
 * Trabajo para la asignatura TAIA
 * Practica individual con agentes en la plataforma JADE 
 */
package taia.pli.behaviours;

import jade.core.behaviours.Behaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

/**
 * Simulates directing a band
 * 
 * It sends messages to the different members telling them what
 * to play
 * 
 * @author Alex
 *
 */
@SuppressWarnings("serial")
public class DirectBandBehaviour extends Behaviour
{
	//Song variants
	private static final int VARIANTS = 3;
	//Duration of a variant
	private static final int VARIANT_TIME = 2;
	//Ticks the behaviour will count before terminating
	private static final int STEP_NUM = VARIANTS*VARIANT_TIME;

	private int step;
	
	public DirectBandBehaviour()
	{
		
	}
	
	public void onStart()
	{
		step = 0;
	}
	
	@Override
	public void action()
	{
		//Receive special TICK2 message 
		MessageTemplate mt = MessageTemplate.and(MessageTemplate.and(
				MessageTemplate.MatchPerformative(ACLMessage.INFORM),
				MessageTemplate.MatchSender(myAgent.getAID())),
				MessageTemplate.MatchContent(MetronomeBehaviour.TICK+"2"));
		
		ACLMessage msg = myAgent.receive(mt);
				
		if(msg != null)
		{
			//-Walk- a step of this behaviour
			//I cheated to make to steps feel like a single one, otherwise the song
			//would play to fast, and the listener wouldnt be able to identify
			//every part easily
			switch(step)
			{			
				case 1:				
				case 2:
					myAgent.addBehaviour(new SendDirectionBehaviour(0, 1));
					myAgent.addBehaviour(new SendDirectionBehaviour(1, 1));
					myAgent.addBehaviour(new SendDirectionBehaviour(2, 1));
					break;
					
				case 3:
				case 4:
					myAgent.addBehaviour(new SendDirectionBehaviour(0, 2));
					myAgent.addBehaviour(new SendDirectionBehaviour(1, 2));
					myAgent.addBehaviour(new SendDirectionBehaviour(2, 2));
					break;
					
				case 5:					
				case 6:
					myAgent.addBehaviour(new SendDirectionBehaviour(0, 1));
					myAgent.addBehaviour(new SendDirectionBehaviour(1, 2));
					myAgent.addBehaviour(new SendDirectionBehaviour(2, 2));
					break;
					
				default:
					break;
			}
			
			step++;
		}		
	}

	@Override
	public boolean done()
	{
		return step == STEP_NUM;
	}

	public int onEnd()
	{
		super.reset();
		return 0;
	}
}
