/** Copyright (C) 2013 Alexander Mariel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Autor: Alexander Mariel
 * Trabajo para la asignatura TAIA
 * Practica individual con agentes en la plataforma JADE 
 */
package taia.pli.behaviours;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.TickerBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

import java.util.Iterator;

import taia.pli.agents.DrumsAgent;
import taia.pli.agents.LeadAgent;
import taia.pli.utilities.AgentSearcher;
import taia.pli.utilities.DFRegistrar;

/**
 * 
 * This behaviour sends a message to all musicians every X seconds.
 * 
 * This serves as metronome for all the musicians.
 * 
 * @author Alex
 *
 */
@SuppressWarnings("serial")
public class MetronomeBehaviour extends TickerBehaviour
{
	//Constant for the TICK message
	public static final String TICK = "tick";
		
	public MetronomeBehaviour(Agent agent)
	{
		super(agent, DrumsAgent.MAGIC_NUM);
	}
	
	/**
	 * Start when the leader says so
	 */
	public void onStart()
	{
		MessageTemplate mt = 
				MessageTemplate.and(MessageTemplate.MatchPerformative(ACLMessage.INFORM)
						, MessageTemplate.MatchContent(LeadAgent.START));
				
		ACLMessage msg = myAgent.blockingReceive(mt);
		if (msg != null)
		{
			System.out.println(myAgent.getLocalName() + ": Vamos a darle canya!");
		}
	}
	
	/**
	 * Send message on every timer tick
	 */
	@Override
	protected void onTick()
	{				
		ACLMessage cfp = new ACLMessage(ACLMessage.INFORM);
		
		//Get all musician agents
		Iterator<AID> itr = AgentSearcher.getMyDFS().getAgentByDF(myAgent, DFRegistrar.MUSICO).iterator();
		
		//Set every musician as message receiver
		while(itr.hasNext())
		{
			cfp.addReceiver(itr.next());
		}
				
		cfp.setContent(TICK);
		myAgent.send(cfp);
	}

}
