/** Copyright (C) 2013 Alexander Mariel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Autor: Alexander Mariel
 * Trabajo para la asignatura TAIA
 * Practica individual con agentes en la plataforma JADE 
 */
package taia.pli.behaviours;

import jade.core.behaviours.Behaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import taia.pli.agents.LeadAgent;
import taia.pli.utilities.AgentSearcher;
import taia.pli.utilities.DFRegistrar;

/**
 * Wait for every agent to say its ready and then
 * notify the drummer so that he knows he is able to start
 * drumming (and serving as metronome for the rest)
 * 
 * @author Alex
 *
 */
@SuppressWarnings("serial")
public class NotifyStartPerformanceBehaviour extends Behaviour
{
	//Number of ready messages that must be received before telling the drummer
	//to start
	private static final int CONFIRMACIONES_NECESARIAS = 3;
	
	private int receivedConfirmations;
	
	public NotifyStartPerformanceBehaviour()
	{
		
	}
	
	public void onStart()
	{
		receivedConfirmations = 0;
	}
	
	@Override
	public void action()
	{
		MessageTemplate mt = MessageTemplate.and(
				MessageTemplate.MatchPerformative(ACLMessage.INFORM),
				MessageTemplate.MatchContent(LeadAgent.READY));
		ACLMessage msgRecibido = myAgent.receive(mt);
		if (msgRecibido != null)
		{			
			System.out.println(myAgent.getLocalName() + ": Recibida confirmacin de " + msgRecibido.getSender().getLocalName());
			receivedConfirmations++;			
		}
		
		if(receivedConfirmations == CONFIRMACIONES_NECESARIAS)
		{
			//If enough confirmations have been received tell the drummer he can start
			
			System.out.println(myAgent.getLocalName()+ ": Todos estan listos");
			
			ACLMessage msgEnviar = new ACLMessage(ACLMessage.INFORM);
	    	msgEnviar.setSender(myAgent.getAID());
	    	//Search for the drummer
	    	msgEnviar.addReceiver(AgentSearcher.getMyDFS().getAgentByDF(myAgent, DFRegistrar.DRUMS).getFirst());
	    	msgEnviar.setContent(LeadAgent.START);	    	
	    	myAgent.send(msgEnviar);
		}
	}
	
	public boolean done()
	{
		return receivedConfirmations == CONFIRMACIONES_NECESARIAS;
	}
	
	public int onEnd()
	{
		return super.onEnd();
	}

}
