/** Copyright (C) 2013 Alexander Mariel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Autor: Alexander Mariel
 * Trabajo para la asignatura TAIA
 * Practica individual con agentes en la plataforma JADE 
 */
package taia.pli.behaviours;

import jade.core.AID;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import taia.pli.agents.LeadAgent;
import taia.pli.utilities.AgentSearcher;
import taia.pli.utilities.DFRegistrar;

/**
 * Sends a direction message to a musician
 * 
 * A direction message indicates a musician which audio clip to play next
 * 
 * @author Alex
 *
 */
@SuppressWarnings("serial")
public class SendDirectionBehaviour extends OneShotBehaviour
{

	//Number of agent to receive the direction
	private int receiver;
	//Next note to play for the receiver
	private int nextPart;
	
	/**
	 * 
	 * @param pReceiver Number for receiver agent
	 * @param pNextPart Number of next part to play
	 */
	public SendDirectionBehaviour(int pReceiver, int pNextPart)
	{
		receiver = pReceiver;
		nextPart = pNextPart;
	}
	
	@Override
	public void action()
	{
		ACLMessage msgEnviar = new ACLMessage(ACLMessage.REQUEST);

    	msgEnviar.setSender(myAgent.getAID());

    	String receiverAgentType = null;
    	
    	//Decode receiver number
    	switch(receiver)
    	{
    		case 0:
    			receiverAgentType = DFRegistrar.LEAD;
    			break;
    	
    		case 1:
    			receiverAgentType = DFRegistrar.BASS;
    			break;
    			
    		case 2:
    			receiverAgentType = DFRegistrar.DRUMS;
    			break;
    			
    		default:
    			receiverAgentType = DFRegistrar.LEAD;
    			break;
    	}
    	
    	//Get receiver AID
    	msgEnviar.addReceiver(AgentSearcher.getMyDFS().getAgentByDF(myAgent, receiverAgentType).getFirst());
    	
    	msgEnviar.setContent(LeadAgent.DIRECTION_MSG + ";" + nextPart);
    	
    	myAgent.send(msgEnviar);
    	
    	System.out.println(myAgent.getLocalName() + ": He mandado a " + ((AID)msgEnviar.getAllReceiver().next()).getLocalName()
    			+ " que toque " + nextPart);
	}

}
