/** Copyright (C) 2013 Alexander Mariel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Autor: Alexander Mariel
 * Trabajo para la asignatura TAIA
 * Practica individual con agentes en la plataforma JADE 
 */
package taia.pli.behaviours.setupPerformance;

import jade.core.AID;
import jade.core.behaviours.SequentialBehaviour;
import jade.lang.acl.ACLMessage;
import taia.pli.agents.DrumsAgent;
import taia.pli.agents.LeadAgent;
import taia.pli.utilities.AgentSearcher;
import taia.pli.utilities.DFRegistrar;

/**
 * Goes through a set of steps to simulate the agent setting
 * up its equipment.
 * When the agent has finished doing its thing it notifies
 * the leader through a message
 * 
 * @author Alex
 *
 */
@SuppressWarnings("serial")
public class SetupPerformanceBehaviour extends SequentialBehaviour
{		
	
	public SetupPerformanceBehaviour()
	{
		
	}
	
	/**
	 * Add subehaviours, which in the end are subtasks of getting prepared for the
	 * performance
	 */
	public void onStart()
	{
		System.out.println(myAgent.getAID().getLocalName() + ": Voy a prepararme!");
		
		if(myAgent instanceof DrumsAgent)
		{
			//If its a drummer it has no strings to tune... So lets tell a lame joke
			this.addSubBehaviour(new TellBassistJokeBehaviour());
		}
		else
		{
			this.addSubBehaviour(new TuneStringsBehaviour());
		}
		
		this.addSubBehaviour(new SetupAmplifierBehaviour());
	}
	
	/**
	 * Before terminating the behaviour notify the leader of this fact
	 */
	public int onEnd()
	{
    	ACLMessage msg = new ACLMessage(ACLMessage.INFORM);

    	msg.setSender(myAgent.getAID());
    	
    	//Receiver is searched in the DF based on its function in the group
    	msg.addReceiver(AgentSearcher.getMyDFS().getAgentByDF(myAgent, DFRegistrar.LEAD).getFirst());
    	
    	msg.setContent(LeadAgent.READY);
    	
    	myAgent.send(msg);
    	
    	System.out.println(myAgent.getAID().getLocalName() + ": Ey, " + ((AID) msg.getAllReceiver().next()).getLocalName() + ", estoy listo!");	
		
		return 0;		
	}

}
