/** Copyright (C) 2013 Alexander Mariel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Autor: Alexander Mariel
 * Trabajo para la asignatura TAIA
 * Practica individual con agentes en la plataforma JADE 
 */
package taia.pli.utilities;

import jade.core.AID;
import jade.core.Agent;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;

import java.util.LinkedList;

/**
 * This class can be used to search an agnet in the DF
 * by offered services
 * 
 * This class applies a Singleton pattern
 * 
 * @author Alex
 *
 */
public class AgentSearcher
{

	private static AgentSearcher myDFS;
	
	private AgentSearcher()
	{
		
	}
	
	public static AgentSearcher getMyDFS()
	{
		if(myDFS == null)
		{
			myDFS = new AgentSearcher();
		}
		
		return myDFS;
	}
	
	/**
	 * Given an agentType (or service offered by an agent) get all agents
	 * implementing that service otr type
	 * 
	 * @param pAgent Agent object, needed to access the DFService
	 * @param pAgentType String representing the service to be looked for
	 * @return List containing all agents the DFService returned matching pAgentType
	 */
	public LinkedList<AID> getAgentByDF(Agent pAgent, String pAgentType)
	{
		LinkedList<AID> agents = new LinkedList<>();
		DFAgentDescription[] result = null;
		
		try
		{
			//Search in DF
			DFAgentDescription template = new DFAgentDescription();
			ServiceDescription sd = new ServiceDescription();
			sd.setName(pAgentType);
			template.addServices(sd);
			result = DFService.search(pAgent, template);
		}
		catch (FIPAException e)
		{
			e.printStackTrace();
		}
		
		//Extract DF results and put them in a nice structure, like a LinkedList
		if(result != null)
		{			
			for(DFAgentDescription aux:result)
			{
				agents.add(aux.getName());
			}
		}
		
		return agents;
	}
}
