/** Copyright (C) 2013 Alexander Mariel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Autor: Alexander Mariel
 * Trabajo para la asignatura TAIA
 * Practica individual con agentes en la plataforma JADE 
 */
package taia.pli.utilities;

import jade.core.Agent;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;

/**
 * Offers agents registering and deregistering form the containers
 * DF
 * 
 * This object implements the Singleton pattern.
 * 
 * @author Alex
 *
 */
public class DFRegistrar
{
	//Services that can be offered
	public static final String LEAD = "lider";
	public static final String BASS = "bajista";
	public static final String DRUMS = "bateria";
	public static final String MUSICO = "musico";
	
	private static DFRegistrar myDFR;
	
	private DFRegistrar()
	{
		
	}
	
	public static DFRegistrar getMyDFR()
	{
		if(myDFR == null)
		{
			myDFR = new DFRegistrar();
		}
		
		return myDFR;
	}
	
	/**
	 * Registers an agent with the given services 
	 * @param pAgent Agent to be registered
	 * @param pServiceNames Services to be assigned to the agent
	 */
	public void registerInDF(Agent pAgent, String[] pServiceNames)
	{
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(pAgent.getAID());
		
		//Add services to the DFAgentdescription
		for(String aux:pServiceNames)
		{
			ServiceDescription sd = new ServiceDescription();
			sd.setType(aux);
			sd.setName(aux);
			dfd.addServices(sd);
		}	

		try
		{
			//Register agent
			DFService.register(pAgent, dfd);
		}
		catch (FIPAException fe)
		{
			fe.printStackTrace();
		}
	}
	
	/**
	 * Deregister agent from the DF
	 * @param pAgent Agent to be deregistered
	 */
	public void deRegisterFromDF(Agent pAgent)
	{
		try
		{
			DFService.deregister(pAgent);
		}
		catch (FIPAException fe)
		{
			fe.printStackTrace();
		}
	}
}
